@@ -1,4 +1,3 @@
-from .formation_plots import *
 import logging
 import re
 import io
@@ -13,6 +12,7 @@ import matplotlib.dates as mdates
 import xlsxwriter
 from bmxp.gravity import spearman, pearson
 from bmxp import FMDATA, IMDATA
+from .formation_plots import *
 
 matplotlib.use("agg")
 
@@ -20,7 +20,7 @@ logging.basicConfig()
 LOGGER = logging.getLogger(__name__)
 LOGGER.setLevel(logging.INFO)
 
-__version__ = "0.2.13"
+__version__ = "0.3.4"
 
 
 def parse_formatted(dataset):
@@ -97,6 +97,7 @@ def report(
     sample_names=None,
     out_filepath=None,
     write_pdf=True,
+    warnings=None,
 ):
     """
     Handles /formation endpoint
@@ -124,9 +125,9 @@ def report(
         "HMDB_ID": "",
         "Metabolite": "",
     }
-    for fill_key in fill_values:
-        if fill_key not in fmdata.columns:
-            fmdata[fill_key] = fill_values[fill_key]
+    for key, value in fill_values.items():
+        if key not in fmdata.columns:
+            fmdata[key] = value
 
     fmdata.columns.name = None
 
@@ -140,9 +141,9 @@ def report(
         "injection_order": list(range(len(smdata))),
         "column_number": 1,
     }
-    for fill_key in fill_values:
+    for fill_key, value in fill_values.items():
         if fill_key not in smdata.columns:
-            smdata[fill_key] = fill_values[fill_key]
+            smdata[fill_key] = value
 
     # fill necessary missing values
     for sample_type in ("injection_type", "sample_type"):
@@ -206,15 +207,17 @@ def report(
         plt.axis("off")
         plt.text(
             0.5,
-            1,
+            0.9,
             f"QC Report for {dataset_name}",
             ha="center",
             va="top",
             fontsize=24,
+            wrap=True,
+            in_layout=False,
         )
         plt.text(
             0.5,
-            0.9,
+            0.8,
             f"Number of samples: {len(sample_data.columns)}",
             ha="center",
             va="top",
@@ -222,7 +225,7 @@ def report(
         )
         plt.text(
             0.5,
-            0.8,
+            0.7,
             f"Number of features: {len(fmdata.index)}",
             ha="center",
             va="top",
@@ -235,12 +238,30 @@ def report(
                 date = "Unknown"
             plt.text(
                 0.5,
-                0.70,
+                0.6,
                 ("Date of first injection: " + date),
                 ha="center",
                 va="top",
                 fontsize=18,
             )
+        if warnings:
+            plt.text(
+                0,
+                0,
+                "Warnings:\n"
+                + "\n\n".join(
+                    [
+                        warning[:300] + "..." if len(warning) > 300 else warning
+                        for warning in warnings
+                    ]
+                ),
+                ha="left",
+                va="bottom",
+                fontsize=16,
+                wrap=True,
+                in_layout=False,
+            )
+        plt.tight_layout()
         pdf.savefig()
         plt.close("all")
         # create zoomable PCA plot
@@ -490,17 +511,30 @@ def report_from_formatted(
 
 
 def _sort_dataset(final_dataset):
-    """Sort annotated metabolites by order, then, class MZ, RT;
-    sort nontargeted by RT, MZ"""
-    # force internal standards sort to top
-    superclass_present = "superClass" in final_dataset
-    final_dataset.loc[
-        final_dataset["HMDB_ID"].str.lower() == "internal standard", "superClass"
-    ] = "AAAAAAAAAAAAAA"
-    # make sure all annotated features are above nontargeted features
-    final_dataset = final_dataset.sort_values(by=["Metabolite"])
+    """
+    Sort annotated metabolites by order, then class, MZ, RT; sort nontargeted by RT, MZ
+
+    "orderNum" overrides all sorting rules except internal standard sorting; to skip
+    internal standard sorting exclude "HMDB_ID" column or don't label internal standards
+    """
+    ordernum_present = "orderNum" in final_dataset
+    if not ordernum_present:
+        final_dataset["orderNum"] = np.nan
+    # sort ordered or annotated features to top, then sort nontargeted by RT, MZ
+    final_dataset = final_dataset.sort_values(by=["orderNum", "Metabolite", "RT", "MZ"])
+    annotated = ~pd.isnull(final_dataset["Metabolite"]) | ~pd.isnull(
+        final_dataset["orderNum"]
+    )
+
+    # force internal standards to top, missing orderNum next
+    if "HMDB_ID" in final_dataset:
+        final_dataset.loc[
+            final_dataset["HMDB_ID"].str.lower() == "internal standard", "orderNum"
+        ] = -99
+    final_dataset.loc[annotated, "orderNum"] = final_dataset.loc[
+        annotated, "orderNum"
+    ].fillna(-98)
 
-    annotated = ~pd.isnull(final_dataset["Metabolite"])
     # sort annotated if present
     try:
         final_dataset.loc[annotated, :] = (
@@ -519,26 +553,24 @@ def _sort_dataset(final_dataset):
                     if not pd.api.types.is_string_dtype(col.fillna(""))
                     else col.str.lower()
                 ),
+                na_position="first",
             )
             .values
         )
     except KeyError:
         pass
 
-    # sort nontargeted separately
-    final_dataset.loc[~annotated, :] = (
-        final_dataset.loc[~annotated, :].sort_values(by=["RT", "MZ"]).values
-    )
-    if not superclass_present:
-        del final_dataset["superClass"]
+    if not ordernum_present:
+        del final_dataset["orderNum"]
 
     return final_dataset
 
 
 def harmonize_metadata(data, injectionset, sampleset):
-    """Generates combined Sampleset and Injectionset metadata with columns matching
-    data. Also removes not_used. If there are samples in data not present in either
-    sampleset or injectionset, they are preserved and missing metadata is null.
+    """
+    Generates combined Sampleset and Injectionset metadata with columns matching data.
+    If there are samples in data not present in either sampleset or injectionset, they
+    are preserved and missing metadata is null.
     """
     warnings = []
     # rename column for older datasets
@@ -565,6 +597,17 @@ def harmonize_metadata(data, injectionset, sampleset):
     pools = combined.index[combined["broad_id"].str.lower() == "pref"]
     missing = pd.isnull(combined["program_id"])
     if "reporting_name" in combined.columns:
+        mismatches = combined.loc[
+            (combined["program_id"] != combined["reporting_name"])
+            & ~pd.isnull(combined["program_id"])
+            & ~pd.isnull(combined["reporting_name"])
+        ]
+        if len(mismatches) > 0:
+            warnings.append(
+                "There are mismatches between program_ids and reporting_names. Please "
+                + "review and confirm program_ids for the following samples before "
+                + f"sharing results: {', '.join(mismatches.index)}"
+            )
         combined.loc[missing, "program_id"] = combined.loc[missing, "reporting_name"]
     # name pools if possible
     elif pools.str.contains("PREF").all():
@@ -583,12 +626,13 @@ def harmonize_metadata(data, injectionset, sampleset):
 
     qcrole = IMDATA["QCRole"]
     qcrole_map = {
-        "tube_blank": "Tube_Blank",
-        "blank": "Blank",
-        "not_used": "Not_Used",
-        "mm": "Master_Mix",
-        "ms2": "MS2",
-        "brpp": "BRPP",
+        "tube_blank": "QC-Tube_Blank",
+        "blank": "QC-Blank",
+        "not_used": "QC-Not_Used",
+        "mm": "QC-Master_Mix",
+        "ms2": "QC-MS2",
+        "brpp": "QC-BRPP",
+        "bridge_pref": "QC-Scaling_Pool",
     }
 
     if qcrole in combined.columns:
@@ -635,7 +679,6 @@ def harmonize_metadata(data, injectionset, sampleset):
 
 def fill_f_mdata(metadata, annotations):
     """combine feature metadata from dataset and database queries for final results"""
-    # request annotations and their HMBD IDs
     main_cols = ["__annotation_id", "Compound_ID", "MZ", "RT"]
     additional_meta = [col for col in metadata.columns if col not in main_cols]
     feature_meta = metadata.loc[:, main_cols + additional_meta]
@@ -660,9 +703,6 @@ def fill_f_mdata(metadata, annotations):
         to_fill = feature_meta["__annotation_id"] == key
         for anno_key in value:
             feature_meta.loc[to_fill, anno_key] = value[anno_key]
-
-    # format labels for each column; None = default formatting
-    # feature_meta = feature_meta.reset_index(drop=True)
     return feature_meta
 
 
@@ -775,50 +815,56 @@ def filter_samples_mask(data, smdata, formation_params):
     return to_keep, warnings
 
 
-def filter_features_mask(data, smdata, fmdata, form_params):
+def filter_features_mask(data, imdata, fmdata, form_params):
     """
     Generates a filtering boolean key for data
     """
     warnings = []
     to_keep = np.array([True] * len(fmdata))
-    if "Primary" in fmdata.columns:
-        to_keep = to_keep & fmdata["Primary"].fillna(False)
+    ann_id_col = FMDATA["Annotation_ID"]
+    adduct_col = FMDATA["Adduct"]
+    inj_type_col = IMDATA["injection_type"]
 
     if not fmdata.index.equals(data.index):
         raise IndexError("Your Feature Metadata does not match your data index.")
 
-    pref_as = smdata.index[smdata["injection_type"] == "prefa"]
-    pref_bs = smdata.index[smdata["injection_type"] == "prefb"]
-    miss_column = f"PREF Missing (of {len(pref_as) + len(pref_bs)})"
-    if form_params["missing_as_percent"]:
-        missing_cutoff = np.ceil(
-            (len(pref_as) + len(pref_bs)) * form_params["missing_cutoff"] / 100
-        )
-    else:
-        missing_cutoff = form_params["missing_cutoff"]
+    pref_as = imdata.index[imdata[inj_type_col].str.startswith("prefa", na=False)]
+    pref_bs = imdata.index[imdata[inj_type_col].str.startswith("prefb", na=False)]
 
-    to_keep = to_keep & (fmdata[miss_column] <= missing_cutoff)
-    if len(pref_as) > 0:
-        to_keep = to_keep & (fmdata["PREFA CVs"] <= form_params["cv_cutoff"] / 100)
-    if len(pref_bs) > 0:
-        to_keep = to_keep & (fmdata["PREFB CVs"] <= form_params["cv_cutoff"] / 100)
+    if form_params.get("filter_by_pref_missing", False):
+        miss_column = f"PREF Missing (of {len(pref_as) + len(pref_bs)})"
+        if form_params.get("missing_as_percent", False):
+            missing_cutoff = np.ceil(
+                (len(pref_as) + len(pref_bs)) * form_params["missing_cutoff"] / 100
+            )
+        else:
+            missing_cutoff = form_params["missing_cutoff"]
+        to_keep = to_keep & (fmdata[miss_column] <= missing_cutoff)
 
-    if "Primary" in fmdata.columns:
+    if form_params.get("filter_by_pref_cv", False):
+        if len(pref_as) > 0:
+            to_keep = to_keep & (fmdata["PREFA CVs"] <= form_params["cv_cutoff"] / 100)
+        if len(pref_bs) > 0:
+            to_keep = to_keep & (fmdata["PREFB CVs"] <= form_params["cv_cutoff"] / 100)
+
+    if form_params.get("filter_by_clusters", False) and "Primary" in fmdata.columns:
         to_keep = to_keep & fmdata["Primary"].fillna(False)
 
     # annotated compounds don't get filtered by CV or missing PREFs
-    to_keep = to_keep | pd.notnull(fmdata["Annotation_ID"])
+    to_keep = to_keep | pd.notnull(fmdata[ann_id_col])
 
     # but everything by non_quant
-    non_quant = fmdata["Non_Quant"].fillna(False).astype(bool)
-    to_keep = to_keep & ~non_quant
+    if form_params.get("filter_by_nonquant", False):
+        non_quant = fmdata[FMDATA["Non_Quant"]].fillna(False).astype(bool)
+        to_keep = to_keep & ~non_quant
 
     # find all duplicated annotations that weren't filtered as non_quant
-    annotated = to_keep & pd.notnull(fmdata["Annotation_ID"])
-    is_duplicated = fmdata.loc[annotated, "Annotation_ID"].duplicated(keep=False)
+    annotated = to_keep & pd.notnull(fmdata[ann_id_col])
+    is_duplicated = fmdata.loc[annotated, ann_id_col].duplicated(keep=False)
     duplicates = fmdata.loc[is_duplicated.index[is_duplicated]]
-    if "Adduct" in fmdata.columns and not form_params["keep_adducts"] == "all":
-        if form_params["keep_adducts"] == "top":
+    keep_adducts = form_params.get("keep_adducts", "all")
+    if adduct_col in fmdata.columns and keep_adducts != "all":
+        if keep_adducts == "top":
             # only keep annotation if top priority adduct
             fmdata_adduct_priority_null_vals = (
                 fmdata["Adduct_Priority"].fillna(False).astype(bool)
@@ -827,46 +873,52 @@ def filter_features_mask(data, smdata, fmdata, form_params):
             fmdata_top_adduct = fmdata_adduct_priority_defined["Adduct_Priority"].map(
                 lambda x: x.split(",")[0]
             )
-            to_drop = fmdata_adduct_priority_defined["Adduct"] != fmdata_top_adduct
+            to_drop = fmdata_adduct_priority_defined[adduct_col] != fmdata_top_adduct
             to_drop_idx = to_drop.index[to_drop]
             to_keep[to_drop_idx] = False
         else:
-            for anno in set(duplicates["Annotation_ID"]):
-                group = duplicates.loc[duplicates["Annotation_ID"] == anno]
+            for anno in set(duplicates[ann_id_col]):
+                group = duplicates.loc[duplicates[ann_id_col] == anno]
                 # pick highest priority adduct, if possible
                 priorities = group.loc[group.index[0], "Adduct_Priority"]
                 if not priorities:
                     continue
                 priorities = priorities.split(",")
                 for adduct in priorities:
-                    if adduct in group["Adduct"].values:
-                        to_drop = group["Adduct"] != adduct
+                    if adduct in group[adduct_col].values:
+                        to_drop = group[adduct_col] != adduct
                         to_drop_idx = to_drop.index[to_drop]
                         to_keep[to_drop_idx] = False
                         break
 
     # find annotations that are still duplicated
-    annotated = to_keep & pd.notnull(fmdata["Annotation_ID"])
-    is_duplicated = fmdata.loc[annotated, "Annotation_ID"].duplicated(keep=False)
-    duplicates = fmdata.loc[is_duplicated.index[is_duplicated]]
+    if form_params.get("filter_by_extraction_method", False):
+        annotated = to_keep & pd.notnull(fmdata[ann_id_col])
+        is_duplicated = fmdata.loc[annotated, ann_id_col].duplicated(keep=False)
+        duplicates = fmdata.loc[is_duplicated.index[is_duplicated]]
 
-    for anno in set(duplicates["Annotation_ID"]):
-        group = duplicates.loc[duplicates["Annotation_ID"] == anno]
-        # drop QI annotation(s) if there is a TF annotation
-        if form_params["feature_priority"][0] in group["__extraction_method"].values:
-            to_drop = group["__extraction_method"] == form_params["feature_priority"][1]
-            to_drop_idx = to_drop.index[to_drop]
-            to_keep[to_drop_idx] = False
+        for anno in set(duplicates[ann_id_col]):
+            group = duplicates.loc[duplicates[ann_id_col] == anno]
+            # drop QI annotation(s) if there is a TF annotation
+            if (
+                form_params["feature_priority"][0]
+                in group["__extraction_method"].values
+            ):
+                to_drop = (
+                    group["__extraction_method"] == form_params["feature_priority"][1]
+                )
+                to_drop_idx = to_drop.index[to_drop]
+                to_keep[to_drop_idx] = False
 
     # finally, if there are still duplicates, warn the user
-    annotated = to_keep & pd.notnull(fmdata["Annotation_ID"])
-    cols_to_check = ["Annotation_ID"]
-    if "Adduct" in fmdata.columns and form_params["keep_adducts"] == "all":
-        cols_to_check.append("Adduct")
+    annotated = to_keep & pd.notnull(fmdata[ann_id_col])
+    cols_to_check = [ann_id_col]
+    if adduct_col in fmdata.columns and keep_adducts == "all":
+        cols_to_check.append(adduct_col)
     is_duplicated = fmdata.loc[annotated].duplicated(subset=cols_to_check, keep=False)
     if sum(is_duplicated) > 0:
         warnings.append(
-            "There are duplicate annotations that could not be filtered by adduct "
+            "There are duplicate annotations that were not filtered by adduct "
             + "or extraction method. Please remove duplicate annotations before "
             + "sharing results."
         )
@@ -874,12 +926,13 @@ def filter_features_mask(data, smdata, fmdata, form_params):
     return to_keep.values, warnings
 
 
-def feature_qc(data, smdata, fmdata):
+def feature_qc(data, imdata, fmdata):
     """
     Generates QC data and adds to feature metadata
     """
-    pref_as = smdata.index[smdata["injection_type"] == "prefa"]
-    pref_bs = smdata.index[smdata["injection_type"] == "prefb"]
+    inj_type_col = IMDATA["injection_type"]
+    pref_as = imdata.index[imdata[inj_type_col].str.startswith("prefa", na=False)]
+    pref_bs = imdata.index[imdata[inj_type_col].str.startswith("prefb", na=False)]
     miss_column = f"PREF Missing (of {len(pref_as) + len(pref_bs)})"
 
     if len(pref_as) > 0:
@@ -898,19 +951,34 @@ def feature_qc(data, smdata, fmdata):
     return fmdata, miss_column
 
 
-def combine(data, smdata, fmdata, fmdata_formats):
-    """Returns a formatted dataset and any warnings"""
-    warnings = []
-    abundance_threshold = 1
-    fmdata = fmdata.copy()
-    smdata = smdata.copy()
+def prepare_sample_metadata(data, smdata):
+    """
+    Transposes sample metadata, aligns it with dataset columns, and creates column
+    header DataFrame with program_ids
+    """
+    smdata = smdata.copy().T
+    headers = data.columns.to_frame().T
+    headers.loc[0, smdata.columns] = smdata.loc["program_id"]
+    smdata = smdata.drop(index="program_id")
+    smdata["Metabolite"] = smdata.index.str.capitalize()
+    smdata = smdata.reindex(columns=data.columns, fill_value="")
+    return smdata, headers
 
-    # check mdata and data have same index
+
+def combine_feature_metadata(data, fmdata, fmdata_formats, abundance_threshold=1):
+    """
+    Combines feature metadata and abundances into a single DataFrame, rounds data, and
+    sorts features
+    """
+    warnings = []
+    fmdata = fmdata.copy()
+
+    # check fmdata and data have same index
     dataset_index = data.index
     if set(fmdata.index) != set(data.index):
         warnings.append(
             "Your metadata and data have mismatched features, likely because this"
-            " is an old dataset. You might have missing features..."
+            " is an old dataset. You might have missing features."
         )
         dataset_index = fmdata.index.intersection(data.index).copy()
         data = data.loc[dataset_index, :]
@@ -919,46 +987,39 @@ def combine(data, smdata, fmdata, fmdata_formats):
     # sort and drop index (which was Compound_ID); Compound_ID should be in feature metadata
     fmdata = fmdata.loc[dataset_index, :].reset_index(drop=True)
     data.reset_index(drop=True, inplace=True)
-    data[data < abundance_threshold] = np.nan
-
     # fill non_quants and cast as object
     if "Non_Quant" not in fmdata.columns:
         fmdata["Non_Quant"] = False
     fmdata.fillna({"Non_Quant": False}, inplace=True)
     fmdata["Non_Quant"] = fmdata["Non_Quant"].astype("object")
-
-    # fill values less than 1 with 1 and round
-    data = data.clip(lower=1).fillna(0)
-    data = data.round(0).astype(np.int64)
-    data = data.replace(0, np.nan)
-
+    # remove values below abundance_threshold and round
+    data[data < abundance_threshold] = np.nan
+    data = data.round(0)
+    data.replace(0, np.nan, inplace=True)
     # keep additional columns but drop columns that are for internal use only
+    fmdata_cols = [col for col in fmdata_formats if col in fmdata.columns]
     additional_cols = [
-        col for col in fmdata if col not in fmdata_formats and not col.startswith("__")
+        col for col in fmdata if col not in fmdata_cols and not col.startswith("__")
     ]
-    fmdata = fmdata.reindex(columns=additional_cols + list(fmdata_formats))
+    fmdata = fmdata.reindex(columns=additional_cols + fmdata_cols)
+    combined_dataset = pd.concat([fmdata, data], axis="columns", copy=False)
+    combined_dataset = _sort_dataset(combined_dataset)
+    combined_dataset = combined_dataset.reset_index(drop=True).reset_index()
 
-    final_dataset = pd.concat([fmdata, data], axis="columns")
-    final_dataset = _sort_dataset(final_dataset)
-
-    # set headers as top row
-    injection_meta = smdata.T
-    final_dataset = final_dataset.reset_index(drop=True).reset_index()
-    final_dataset = final_dataset.astype("object")
-    final_dataset.loc[-1, final_dataset.columns] = final_dataset.columns
-    final_dataset.loc[-1, injection_meta.columns] = injection_meta.loc["program_id"]
-    final_dataset = final_dataset.sort_index()
-    final_dataset = final_dataset.rename(index={-1: "Metabolite"})
-    injection_meta = injection_meta.drop(index="program_id")
-    injection_meta.loc[:, "Metabolite"] = injection_meta.index.str.capitalize()
-    original_columns = final_dataset.columns
-    final_dataset = pd.concat([injection_meta, final_dataset], axis="index")
-    final_dataset = final_dataset.loc[:, original_columns]
-    final_dataset = final_dataset.fillna("")
-    return final_dataset, warnings
+    return combined_dataset, warnings
 
 
-def to_excel(data, fmdata_formats, sdmata_formats, miss_column, method_name="Default"):
+def to_excel(
+    data,
+    smdata,
+    headers,
+    fmdata_formats,
+    smdata_formats,
+    column_widths=None,
+    method_name="Default",
+    smformat_default="center",
+    fmformat_default="header_center",
+):
     """
     Returns a BytesIO object of a formatted excel sheet
     """
@@ -966,6 +1027,12 @@ def to_excel(data, fmdata_formats, sdmata_formats, miss_column, method_name="Def
     output = io.BytesIO()
     workbook = xlsxwriter.Workbook(output)
     worksheet = workbook.add_worksheet(method_name)
+    worksheet.add_write_handler(
+        float,
+        lambda worksheet, row, col, val, fmt=None: (
+            None if not pd.isnull(val) else worksheet.write_blank(row, col, None, fmt)
+        ),
+    )
     formats = {
         "date": workbook.add_format(
             {
@@ -1014,7 +1081,9 @@ def to_excel(data, fmdata_formats, sdmata_formats, miss_column, method_name="Def
     column_formats = {
         header: formats[label] for header, label in fmdata_formats.items()
     }
-    row_formats = {index: formats[label] for index, label in sdmata_formats.items()}
+    row_formats = {index: formats[label] for index, label in smdata_formats.items()}
+    smformat_default = formats[smformat_default]
+    fmformat_default = formats[fmformat_default]
 
     # column widths
     worksheet.set_column_pixels(
@@ -1037,11 +1106,6 @@ def to_excel(data, fmdata_formats, sdmata_formats, miss_column, method_name="Def
         data.columns.get_loc("subClass"),
         80,
     )
-    worksheet.set_column_pixels(
-        data.columns.get_loc(miss_column),
-        data.columns.get_loc(miss_column),
-        80,
-    )
     worksheet.set_column_pixels(
         data.columns.get_loc("Annotation_ID"),
         data.columns.get_loc("Annotation_ID"),
@@ -1072,29 +1136,48 @@ def to_excel(data, fmdata_formats, sdmata_formats, miss_column, method_name="Def
         len(data.columns) - 1,
         150,
     )
+    if column_widths is not None:
+        for col_name, width in column_widths.items():
+            if col_name in data.columns:
+                worksheet.set_column_pixels(
+                    data.columns.get_loc(col_name),
+                    data.columns.get_loc(col_name),
+                    width,
+                )
 
-    for i, row in enumerate(data.values):
-        index_label = data.index[i]
-        if index_label in row_formats.keys():  # injection metadata
+    for i, row in enumerate(smdata.values):
+        index_label = smdata.index[i]
+        if index_label in row_formats:
             worksheet.set_row_pixels(i, 16, row_formats[index_label])
-        else:  # default to int
-            worksheet.set_row_pixels(i, 16, formats["int"])
-
+        else:
+            worksheet.set_row_pixels(i, 16, smformat_default)
         for j, val in enumerate(row):
-            column_label = data.columns[j]
-            # metadata headers
-            if index_label == "Metabolite" and column_label in column_formats.keys():
-                worksheet.write(i, j, val, formats["header"])
-            # data headers
-            elif index_label == "Metabolite":
-                worksheet.write(i, j, val, formats["header_center"])
-            # row labels
-            elif index_label in row_formats.keys() and column_label == "Metabolite":
+            if data.columns[j] == "Metabolite":  # row labels
                 worksheet.write(i, j, val, formats["label"])
-            # feature metadata
-            elif column_label in column_formats.keys():
-                worksheet.write(i, j, val, column_formats[column_label])
+            elif pd.isnull(val):
+                # replace with a write handler for dates if we add more handlers later
+                worksheet.write_blank(i, j, None)
             else:
                 worksheet.write(i, j, val)
+
+    row_offset = len(smdata)
+    worksheet.set_row_pixels(row_offset, 16, formats["int"])
+    for j, val in enumerate(headers.values[0]):
+        if data.columns[j] in column_formats:  # metadata headers
+            worksheet.write(row_offset, j, val, formats["header"])
+        else:  # data headers
+            worksheet.write(row_offset, j, val, fmformat_default)
+
+    row_offset += 1
+    for i, row in enumerate(data.values):
+        index_label = data.index[i]
+        worksheet.set_row_pixels(i + row_offset, 16, formats["int"])
+        for j, val in enumerate(row):
+            column_label = data.columns[j]
+            if column_label in column_formats:  # feature metadata
+                worksheet.write(i + row_offset, j, val, column_formats[column_label])
+            else:
+                worksheet.write(i + row_offset, j, val)
+
     workbook.close()
     return output
